@@ -1,12 +1,12 @@
 import time
 
 _this_year = time.strftime("%Y")
-__version__ = '1.3.7post0'
-__author__ = 'William Falcon et al.'
-__author_email__ = 'waf2107@columbia.edu'
-__license__ = 'Apache-2.0'
-__copyright__ = f'Copyright (c) 2018-{_this_year}, {__author__}.'
-__homepage__ = 'https://github.com/PyTorchLightning/pytorch-lightning'
+__version__ = "1.4.0"
+__author__ = "William Falcon et al."
+__author_email__ = "waf2107@columbia.edu"
+__license__ = "Apache-2.0"
+__copyright__ = f"Copyright (c) 2018-{_this_year}, {__author__}."
+__homepage__ = "https://github.com/PyTorchLightning/pytorch-lightning"
 __docs_url__ = "https://pytorch-lightning.readthedocs.io/en/stable/"
 # this has to be simple string, see: https://github.com/pypa/twine/issues/522
 __docs__ = (
@@ -35,12 +35,4 @@ Documentation
 - https://pytorch-lightning.readthedocs.io/en/stable
 """
 
-__all__ = [
-    "__author__",
-    "__author_email__",
-    "__copyright__",
-    "__docs__",
-    "__homepage__",
-    "__license__",
-    "__version__",
-]
+__all__ = ["__author__", "__author_email__", "__copyright__", "__docs__", "__homepage__", "__license__", "__version__"]
