@@ -76,8 +76,8 @@ class BasePredictionWriter(Callback):
 
     def write_on_batch_end(
         self,
-        trainer: 'pl.Trainer',
-        pl_module: 'pl.LightningModule',
+        trainer: "pl.Trainer",
+        pl_module: "pl.LightningModule",
         prediction: Any,
         batch_indices: Optional[Sequence[int]],
         batch: Any,
@@ -89,8 +89,8 @@ class BasePredictionWriter(Callback):
 
     def write_on_epoch_end(
         self,
-        trainer: 'pl.Trainer',
-        pl_module: 'pl.LightningModule',
+        trainer: "pl.Trainer",
+        pl_module: "pl.LightningModule",
         predictions: Sequence[Any],
         batch_indices: Optional[Sequence[Any]],
     ) -> None:
@@ -99,8 +99,8 @@ class BasePredictionWriter(Callback):
 
     def on_predict_batch_end(
         self,
-        trainer: 'pl.Trainer',
-        pl_module: 'pl.LightningModule',
+        trainer: "pl.Trainer",
+        pl_module: "pl.LightningModule",
         outputs: Any,
         batch: Any,
         batch_idx: int,
@@ -109,11 +109,11 @@ class BasePredictionWriter(Callback):
         if not self.interval.on_batch:
             return
         is_distributed = trainer.accelerator_connector.is_distributed
-        batch_indices = trainer.predict_loop.batch_indices if is_distributed else None
+        batch_indices = trainer.predict_loop.epoch_loop.current_batch_indices if is_distributed else None
         self.write_on_batch_end(trainer, pl_module, outputs, batch_indices, batch, batch_idx, dataloader_idx)
 
     def on_predict_epoch_end(
-        self, trainer: 'pl.Trainer', pl_module: 'pl.LightningModule', outputs: Sequence[Any]
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", outputs: Sequence[Any]
     ) -> None:
         if not self.interval.on_epoch:
             return
