@@ -16,6 +16,7 @@ MLflow Logger
 -------------
 """
 import logging
+import os
 import re
 from argparse import Namespace
 from time import time
@@ -30,18 +31,20 @@ _MLFLOW_AVAILABLE = _module_available("mlflow")
 try:
     import mlflow
     from mlflow.tracking import context, MlflowClient
+    from mlflow.utils.mlflow_tags import MLFLOW_RUN_NAME
 # todo: there seems to be still some remaining import error with Conda env
 except ImportError:
     _MLFLOW_AVAILABLE = False
     mlflow, MlflowClient, context = None, None, None
+    MLFLOW_RUN_NAME = "mlflow.runName"
 
 # before v1.1.0
-if hasattr(context, 'resolve_tags'):
+if hasattr(context, "resolve_tags"):
     from mlflow.tracking.context import resolve_tags
 
 
 # since v1.1.0
-elif hasattr(context, 'registry'):
+elif hasattr(context, "registry"):
     from mlflow.tracking.context.registry import resolve_tags
 else:
 
@@ -84,8 +87,11 @@ class MLFlowLogger(LightningLoggerBase):
 
     Args:
         experiment_name: The name of the experiment
+        run_name: Name of the new run. The `run_name` is internally stored as a ``mlflow.runName`` tag.
+            If the ``mlflow.runName`` tag has already been set in `tags`, the value is overridden by the `run_name`.
         tracking_uri: Address of local or remote tracking server.
-            If not provided, defaults to `file:<save_dir>`.
+            If not provided, defaults to `MLFLOW_TRACKING_URI` environment variable if set, otherwise it falls
+            back to `file:<save_dir>`.
         tags: A dictionary tags for the experiment.
         save_dir: A path to a local directory where the MLflow runs get saved.
             Defaults to `./mlflow` if `tracking_uri` is not provided.
@@ -99,29 +105,30 @@ class MLFlowLogger(LightningLoggerBase):
             If required MLFlow package is not installed on the device.
     """
 
-    LOGGER_JOIN_CHAR = '-'
+    LOGGER_JOIN_CHAR = "-"
 
     def __init__(
         self,
-        experiment_name: str = 'default',
-        tracking_uri: Optional[str] = None,
+        experiment_name: str = "default",
+        run_name: Optional[str] = None,
+        tracking_uri: Optional[str] = os.getenv("MLFLOW_TRACKING_URI"),
         tags: Optional[Dict[str, Any]] = None,
-        save_dir: Optional[str] = './mlruns',
-        prefix: str = '',
+        save_dir: Optional[str] = "./mlruns",
+        prefix: str = "",
         artifact_location: Optional[str] = None,
     ):
         if mlflow is None:
             raise ImportError(
-                'You want to use `mlflow` logger which is not installed yet,'
-                ' install it with `pip install mlflow`.'
+                "You want to use `mlflow` logger which is not installed yet, install it with `pip install mlflow`."
             )
         super().__init__()
         if not tracking_uri:
-            tracking_uri = f'{LOCAL_FILE_URI_PREFIX}{save_dir}'
+            tracking_uri = f"{LOCAL_FILE_URI_PREFIX}{save_dir}"
 
         self._experiment_name = experiment_name
         self._experiment_id = None
         self._tracking_uri = tracking_uri
+        self._run_name = run_name
         self._run_id = None
         self.tags = tags
         self._prefix = prefix
@@ -146,13 +153,20 @@ class MLFlowLogger(LightningLoggerBase):
             if expt is not None:
                 self._experiment_id = expt.experiment_id
             else:
-                log.warning(f'Experiment with name {self._experiment_name} not found. Creating it.')
+                log.warning(f"Experiment with name {self._experiment_name} not found. Creating it.")
                 self._experiment_id = self._mlflow_client.create_experiment(
-                    name=self._experiment_name,
-                    artifact_location=self._artifact_location,
+                    name=self._experiment_name, artifact_location=self._artifact_location
                 )
 
         if self._run_id is None:
+            if self._run_name is not None:
+                self.tags = self.tags or {}
+                if MLFLOW_RUN_NAME in self.tags:
+                    log.warning(
+                        f"The tag {MLFLOW_RUN_NAME} is found in tags. "
+                        f"The value will be overridden by {self._run_name}."
+                    )
+                self.tags[MLFLOW_RUN_NAME] = self._run_name
             run = self._mlflow_client.create_run(experiment_id=self._experiment_id, tags=resolve_tags(self.tags))
             self._run_id = run.info.run_id
         return self._mlflow_client
@@ -184,30 +198,31 @@ class MLFlowLogger(LightningLoggerBase):
 
     @rank_zero_only
     def log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None) -> None:
-        assert rank_zero_only.rank == 0, 'experiment tried to log from global_rank != 0'
+        assert rank_zero_only.rank == 0, "experiment tried to log from global_rank != 0"
 
         metrics = self._add_prefix(metrics)
 
         timestamp_ms = int(time() * 1000)
         for k, v in metrics.items():
             if isinstance(v, str):
-                log.warning(f'Discarding metric with string value {k}={v}.')
+                log.warning(f"Discarding metric with string value {k}={v}.")
                 continue
 
             new_k = re.sub("[^a-zA-Z0-9_/. -]+", "", k)
             if k != new_k:
                 rank_zero_warn(
                     "MLFlow only allows '_', '/', '.' and ' ' special characters in metric name."
-                    f" Replacing {k} with {new_k}.", RuntimeWarning
+                    f" Replacing {k} with {new_k}.",
+                    RuntimeWarning,
                 )
                 k = new_k
 
             self.experiment.log_metric(self.run_id, k, v, timestamp_ms, step)
 
     @rank_zero_only
-    def finalize(self, status: str = 'FINISHED') -> None:
+    def finalize(self, status: str = "FINISHED") -> None:
         super().finalize(status)
-        status = 'FINISHED' if status == 'success' else status
+        status = "FINISHED" if status == "success" else status
         if self.experiment.get_run(self.run_id):
             self.experiment.set_terminated(self.run_id, status)
 
