@@ -15,7 +15,6 @@ from weakref import proxy
 
 
 class ModelConnector:
-
     def __init__(self, trainer):
         self.trainer = trainer
 
@@ -24,7 +23,5 @@ class ModelConnector:
 
         for m in [model, ref_model]:
             m.trainer = proxy(self.trainer)
-            m._device_type = str(self.trainer._device_type)
-            m._distrib_type = str(self.trainer._distrib_type)
             m.use_amp = self.trainer.amp_backend is not None
             m.precision = self.trainer.precision
