@@ -35,7 +35,7 @@ def recursive_detach(in_dict: dict, to_cpu: bool = False) -> dict:
     for k, v in in_dict.items():
         if isinstance(v, dict):
             v = recursive_detach(v, to_cpu=to_cpu)
-        elif callable(getattr(v, 'detach', None)):
+        elif callable(getattr(v, "detach", None)):
             v = v.detach()
             if to_cpu:
                 v = v.cpu()
@@ -44,36 +44,46 @@ def recursive_detach(in_dict: dict, to_cpu: bool = False) -> dict:
 
 
 def is_oom_error(exception):
-    return is_cuda_out_of_memory(exception) \
-        or is_cudnn_snafu(exception) \
-        or is_out_of_cpu_memory(exception)
+    return is_cuda_out_of_memory(exception) or is_cudnn_snafu(exception) or is_out_of_cpu_memory(exception)
 
 
 # based on https://github.com/BlackHC/toma/blob/master/toma/torch_cuda_memory.py
 def is_cuda_out_of_memory(exception):
-    return isinstance(exception, RuntimeError) \
-        and len(exception.args) == 1 \
-        and "CUDA out of memory." in exception.args[0]
+    return (
+        isinstance(exception, RuntimeError)
+        and len(exception.args) == 1
+        and "CUDA" in exception.args[0]
+        and "out of memory" in exception.args[0]
+    )
 
 
 # based on https://github.com/BlackHC/toma/blob/master/toma/torch_cuda_memory.py
 def is_cudnn_snafu(exception):
     # For/because of https://github.com/pytorch/pytorch/issues/4107
-    return isinstance(exception, RuntimeError) \
-        and len(exception.args) == 1 \
+    return (
+        isinstance(exception, RuntimeError)
+        and len(exception.args) == 1
         and "cuDNN error: CUDNN_STATUS_NOT_SUPPORTED." in exception.args[0]
+    )
 
 
 # based on https://github.com/BlackHC/toma/blob/master/toma/cpu_memory.py
 def is_out_of_cpu_memory(exception):
-    return isinstance(exception, RuntimeError) \
-        and len(exception.args) == 1 \
+    return (
+        isinstance(exception, RuntimeError)
+        and len(exception.args) == 1
         and "DefaultCPUAllocator: can't allocate memory" in exception.args[0]
+    )
 
 
 # based on https://github.com/BlackHC/toma/blob/master/toma/torch_cuda_memory.py
 def garbage_collection_cuda():
     """Garbage collection Torch (CUDA) memory."""
     gc.collect()
-    if torch.cuda.is_available():
+    try:
+        # This is the last thing that should cause an OOM error, but seemingly it can.
         torch.cuda.empty_cache()
+    except RuntimeError as exception:
+        if not is_oom_error(exception):
+            # Only handle OOM errors
+            raise
